@@ -46,26 +46,6 @@
 #include <string.h>
 #include <ctype.h>
 
-#if HAVE_INTTYPES_H
-# include <inttypes.h>
-#endif
-
-#if ! defined PRIdMAX
-# if HAVE_LONG_LONG
-#  define PRIdMAX "lld"
-# else
-#  define PRIdMAX "ld"
-# endif
-#endif
-
-#if ! defined PRIuMAX
-# if HAVE_LONG_LONG
-#  define PRIuMAX "llu"
-# else
-#  define PRIuMAX "lu"
-# endif
-#endif
-
 #include "system.h"
 
 #include "closeout.h"
